# -*- coding: UTF-8 -*-
"""!urban <term> returns the urban dictionary definition and example of a term"""
import requests
import re
import sys

try:
    from urllib import quote
except ImportError:
    from urllib.request import quote

def reply_quote(string):
    lines = string.split("\n")
    return "\n".join("> _{0}_".format(l.strip()) for l in lines)

def urban(term):
    # slack likes to replace the quote character with a smart quote.
    # Undo that.
    term = term.replace(u'’', "'").encode("utf8")

    baseurl = u"http://api.urbandictionary.com/v0/define?term={0}"
    data = requests.get(baseurl.format(quote(term))).json()

    try:
        result = data['list'][0]
        result["example"] = reply_quote(result.get("example", ""))
        definition = (u"*{word}*: {definition}.\n"
                        "*Example:*\n{example}".format(**result))
        return definition
    except IndexError:
        return ":cry: No results found for {0}, please try again".format(term)

def on_message(msg, server):
    text = msg.get("text", "")
    match = re.findall(r"!urban (.*)", text)
    if not match:
        return
    searchterm = match[0]
    return urban(searchterm)



from .slack import User, Bot

class FakeServer(object):
    def __init__(self, slack=None, config=None, hooks=None, db=None):
        self.slack = slack or FakeSlack()
        self.config = config
        self.hooks = hooks
        self.db = db

    def query(self, sql, *params):
        if not self.db:
            return None

        c = self.db.cursor()
        c.execute(sql, params)
        rows = c.fetchall()
        c.close()
        self.db.commit()
        return rows

class FakeSlack(object):
    def __init__(self, server=None, users=None, events=None, bots=None, botname="test"):
        self.posted_message = None
        self.events = events if events else []
        self.login_data = {
            "self": {
                "name": botname,
            }
        }
        self.username = "limbo_test"
        self.userid = "1"

        self.users = users if users else {
            "1": User(self, "limbo_test", 1, "", 0),
            "2": User(self, "msguser", 2, "", 0),
            "3": User(self, "slackbot", 3, "", 0),
            "4": User(self, "replbot", 4, "", 0),
        }

        self.bots = bots if bots else {
            "1": Bot("1", "otherbot", [], False)
        }


    def post_message(self, channel, message, **kwargs):
        self.posted_message = (message, kwargs)

    def rtm_read(self):
        return self.events.pop() if self.events else []



# -*- coding: UTF-8 -*-
import os
import shlex
import subprocess
import sys

DIR = os.path.dirname(os.path.realpath(__file__))
TESTPLUGINS = os.path.join(DIR, "plugins")

# http://stackoverflow.com/a/13160748/42559
def sh(cmd):
    # slex.split can only handle unicode strings in python3, and only byte
    # strings in python2
    if sys.version_info[0] > 2:
        cmd = cmd.decode("utf8")

    proc = subprocess.Popen(shlex.split(cmd), stdout=subprocess.PIPE)
    output = proc.communicate()[0].decode("utf8")
    ret = proc.returncode
    return output, ret

def test_cmd():
    msg = u"!echo Iñtërnâtiônàlizætiøn"
    out, ret = sh(u"bin/limbo -c '{0}' --pluginpath {1}".format(msg, TESTPLUGINS).encode("utf8"))
    out = out.strip()
    assert out == msg
    assert ret == 0

def test_repl():
    msg = u"!echo Iñtërnâtiônàlizætiøn"
    proc = subprocess.Popen(["bin/limbo", "-t", "--pluginpath", TESTPLUGINS], stdout=subprocess.PIPE, stdin=subprocess.PIPE)
    out = proc.communicate(msg.encode("utf8"))[0]
    out = out.strip().decode("utf8")
    assert out == u"limbo> {0}\nlimbo>".format(msg)
    ret = proc.returncode
    assert ret == 0

#  XXX: TODO
# def test_hook():
#     out, ret = sh(u"limbo -c '' --pluginpath {0} --hook loop".format(TESTPLUGINS))
#     out = out.strip()
#     eq(out, ["init"])
